@@ -6,7 +6,7 @@ from csaps._base import ISmoothingSpline, ISplinePPForm
 from csaps._shortcut import AutoSmoothingResult, csaps
 from csaps._sspndg import NdGridCubicSmoothingSpline, NdGridSplinePPForm
 from csaps._sspumv import CubicSmoothingSpline, SplinePPForm
-from csaps._types import MultivariateDataType, NdGridDataType, UnivariateDataType
+from csaps._types import MultivariateDataType, SequenceUnivariateDataType, UnivariateDataType
 from csaps._version import __version__
 
 __all__ = [
@@ -23,6 +23,6 @@ __all__ = [
     # Type-hints
     'UnivariateDataType',
     'MultivariateDataType',
-    'NdGridDataType',
+    'SequenceUnivariateDataType',
     '__version__',
 ]
