@@ -2,9 +2,7 @@
 ND-Gridded cubic smoothing spline implementation
 """
 
-from typing import Optional, Sequence, Tuple, Union
-import collections.abc as c_abc
-from numbers import Number
+from typing import Optional, Sequence, cast
 
 import numpy as np
 from scipy.interpolate import NdPPoly, PPoly
@@ -18,14 +16,18 @@ from ._reshape import (
     umv_coeffs_to_flatten,
 )
 from ._sspumv import CubicSmoothingSpline
-from ._types import NdGridDataType, UnivariateDataType
+from ._types import Float1DArrayTupe, FloatNDArrayType, SequenceUnivariateDataType
 
 
-def ndgrid_prepare_data_vectors(data, name, min_size: int = 2) -> Tuple[np.ndarray, ...]:
-    if not isinstance(data, c_abc.Sequence):
+def ndgrid_prepare_data_vectors(
+    data: SequenceUnivariateDataType,
+    name: str,
+    min_size: int = 2,
+) -> tuple[Float1DArrayTupe, ...]:
+    if not isinstance(data, Sequence):
         raise TypeError(f"'{name}' must be a sequence of 1-d array-like (vectors) or scalars.")
 
-    data = list(data)
+    data_: list[Float1DArrayTupe] = []
 
     for axis, d in enumerate(data):
         d = np.asarray(d, dtype=np.float64)
@@ -33,12 +35,12 @@ def ndgrid_prepare_data_vectors(data, name, min_size: int = 2) -> Tuple[np.ndarr
             raise ValueError(f"All '{name}' elements must be a vector for axis {axis}.")
         if d.size < min_size:
             raise ValueError(f"'{name}' must contain at least {min_size} data points for axis {axis}.")
-        data[axis] = d
+        data_.append(d)
 
-    return tuple(data)
+    return tuple(data_)
 
 
-class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]], NdPPoly):
+class NdGridSplinePPForm(ISplinePPForm[tuple[np.ndarray, ...], tuple[int, ...]], NdPPoly):
     """N-D grid spline representation in PP-form
 
     N-D grid spline is represented in piecewise tensor product polynomial form.
@@ -53,7 +55,7 @@ class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]],
     __module__ = 'csaps'
 
     @property
-    def breaks(self) -> Tuple[np.ndarray, ...]:
+    def breaks(self) -> tuple[np.ndarray, ...]:
         return self.x
 
     @property
@@ -61,11 +63,11 @@ class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]],
         return self.c
 
     @property
-    def order(self) -> Tuple[int, ...]:
+    def order(self) -> tuple[int, ...]:
         return self.c.shape[: self.c.ndim // 2]
 
     @property
-    def pieces(self) -> Tuple[int, ...]:
+    def pieces(self) -> tuple[int, ...]:
         return self.c.shape[self.c.ndim // 2 :]
 
     @property
@@ -73,13 +75,13 @@ class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]],
         return len(self.x)
 
     @property
-    def shape(self) -> Tuple[int, ...]:
+    def shape(self) -> tuple[int, ...]:
         return tuple(len(xi) for xi in self.x)
 
-    def __call__(
+    def __call__(  # type: ignore[override]
         self,
-        x: Sequence[UnivariateDataType],
-        nu: Optional[Tuple[int, ...]] = None,
+        x: SequenceUnivariateDataType,
+        nu: Optional[tuple[int, ...]] = None,
         extrapolate: Optional[bool] = None,
     ) -> np.ndarray:
         """Evaluate the spline for given data
@@ -87,8 +89,8 @@ class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]],
         Parameters
         ----------
 
-        x : tuple of 1-d array-like
-            The tuple of point values for each dimension to evaluate the spline at.
+        x : Sequence of 1-d array-like
+            The sequence of point values for each dimension to evaluate the spline at.
 
         nu : [*Optional*] tuple of int
             Orders of derivatives to evaluate. Each must be non-negative.
@@ -157,9 +159,9 @@ class NdGridSplinePPForm(ISplinePPForm[Tuple[np.ndarray, ...], Tuple[int, ...]],
 class NdGridCubicSmoothingSpline(
     ISmoothingSpline[
         NdGridSplinePPForm,
-        Tuple[float, ...],
-        NdGridDataType,
-        Tuple[int, ...],
+        tuple[float, ...],
+        SequenceUnivariateDataType,
+        tuple[int, ...],
         bool,
     ]
 ):
@@ -204,31 +206,29 @@ class NdGridCubicSmoothingSpline(
 
     def __init__(
         self,
-        xdata: NdGridDataType,
+        xdata: SequenceUnivariateDataType,
         ydata: np.ndarray,
-        weights: Optional[Union[UnivariateDataType, NdGridDataType]] = None,
-        smooth: Optional[Union[float, Sequence[Optional[float]]]] = None,
+        weights: SequenceUnivariateDataType | None = None,
+        smooth: float | Sequence[float | None] | None = None,
         normalizedsmooth: bool = False,
     ) -> None:
         x, y, w, s = self._prepare_data(xdata, ydata, weights, smooth)
-        coeffs, smooth = self._make_spline(x, y, w, s, normalizedsmooth)
-
-        self._spline = NdGridSplinePPForm.construct_fast(coeffs, x)
-        self._smooth = smooth
+        coeffs, self._smooth = self._make_spline(x, y, w, s, normalizedsmooth)
+        self._spline = cast(NdGridSplinePPForm, NdGridSplinePPForm.construct_fast(coeffs, x))
 
     def __call__(
         self,
-        x: Union[NdGridDataType, Sequence[Number]],
-        nu: Optional[Tuple[int, ...]] = None,
-        extrapolate: Optional[bool] = None,
-    ) -> np.ndarray:
+        x: SequenceUnivariateDataType,
+        nu: tuple[int, ...] | None = None,
+        extrapolate: bool | None = None,
+    ) -> FloatNDArrayType:
         """Evaluate the spline for given data
 
         Parameters
         ----------
 
-        x : tuple of 1-d array-like
-            The tuple of point values for each dimension to evaluate the spline at.
+        x : Sequence of 1-d array-like
+            The sequence of point values for each dimension to evaluate the spline at.
 
         nu : [*Optional*] tuple of int
             Orders of derivatives to evaluate. Each must be non-negative.
@@ -248,12 +248,12 @@ class NdGridCubicSmoothingSpline(
         return self._spline(x, nu=nu, extrapolate=extrapolate)
 
     @property
-    def smooth(self) -> Tuple[float, ...]:
+    def smooth(self) -> tuple[float, ...]:
         """Returns a tuple of smoothing parameters for each axis
 
         Returns
         -------
-        smooth : Tuple[float, ...]
+        smooth : tuple[float, ...]
             The smoothing parameter in the range ``[0, 1]`` for each axis
         """
         return self._smooth
@@ -298,15 +298,13 @@ class NdGridCubicSmoothingSpline(
         if smooth is None:
             smooth = [None] * data_ndim
 
-        if not isinstance(smooth, c_abc.Sequence):
+        if not isinstance(smooth, Sequence):
             smooth = [float(smooth)] * data_ndim
         else:
             smooth = list(smooth)
 
         if len(smooth) != data_ndim:
-            raise ValueError(
-                'Number of smoothing parameter values must ' f'be equal number of dimensions ({data_ndim})'
-            )
+            raise ValueError(f'Number of smoothing parameter values must be equal number of dimensions ({data_ndim})')
 
         return xdata, ydata, weights, smooth
 
